/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007,2008  Arto Karppinen (arto.karppinen@iki.fi),
 * Joni Kokko (t5kojo01@students.oamk.fi),
 * Sami Mäki (kasmra@xob.kapsi.fi),
 * Harri Vattulainen (t5vaha01@students.oamk.fi)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_webdb_client.h"
#include "config.h"

/**
 * @addtogroup IrrecoWebdbClient
 * @ingroup IrrecoWebdb
 *
 * Irreco Web Database API implementation. These functions connect to the
 * IrrecoWebdbClient trough XML-RPC connection, pass arguments, and decode
 * return values.
 *
 * @{
 */

/**
 * @file
 * Source file of @ref IrrecoWebdbClient.
 */

static const char *const value_type[] = {
 "BAD",
 "int",
 "boolean",
 "string",
 "double",
 "datetime",
 "base64",
 "struct",
 "array"
};

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

static void irreco_webdb_client_reset_env();
static SoupXmlrpcResponse *do_xmlrpc(SoupXmlrpcMessage *xmsg,
				     SoupXmlrpcValueType type,
				     IrrecoWebdbClient *self);


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

/**
 * Create new IrrecoWebdbClient Object
 */
IrrecoWebdbClient *irreco_webdb_client_new()
{
	IrrecoWebdbClient *self;
	IRRECO_ENTER

	self = g_slice_new0(IrrecoWebdbClient);

	/* Allocate 1000 characters to soup error message */
	/*self->error_msg = g_malloc0(1000*sizeof(*self->error_msg));*/
	/** @todo do same to do_xmlrpc errormsg gchar */
	/* use GString->str if that can grow on it own*/
	self->error_msg = g_string_new(NULL);

	IRRECO_RETURN_PTR(self);
}

void irreco_webdb_client_free(IrrecoWebdbClient *self)
{
	IRRECO_ENTER

	g_assert(self != NULL);

	/* Free self->error_msg resource */
	g_string_free(self->error_msg, TRUE);
	self->error_msg = NULL;

	g_slice_free(IrrecoWebdbClient, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Private Functions
 * @{
 */

/**
 * Release of RPC results and errors.
 */
static void irreco_webdb_client_reset_env(IrrecoWebdbClient *self)
{
	IRRECO_ENTER

	self->error_msg = g_string_erase(self->error_msg, 0, -1);

	IRRECO_RETURN
}

/** @todo Funcs return bool wether they succeeded, so this could be removed. */
/**
 * Check if the previous call to XMLRPC-C succeeded.
 *
 * @return TRUE if error occured, FALSE otherwise.
 */
/*static gboolean irreco_webdb_client_check_error(IrrecoWebdbClient *self)
{
	IRRECO_ENTER
	if(self->error_env->fault_occurred) {
		IRRECO_ERROR("%d: %s\n", self->error_env->fault_code,
			     self->error_env->fault_string);
		IRRECO_RETURN_BOOL(TRUE);
	}
	IRRECO_RETURN_BOOL(FALSE);
}*/


/**@} */


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */

/**
 * Get error message.
 *
 * @param msg GString object to recieve the error message.
 */
void irreco_webdb_client_get_error_msg(IrrecoWebdbClient *self, GString *msg)
{
	IRRECO_ENTER

	/* Clear and set msg */
	msg = g_string_erase(msg, 0, -1);
	msg = g_string_insert(msg, 0, self->error_msg->str);

	IRRECO_RETURN
}

/**
 * Ask WebDB to sum two numbers.
 *
 * @param num_a	First number.
 * @param num_b Second number.
 * @param sum	Variable to receive the result.
 * @return TRUE on successfull XML-RPC call, FALSE otherwise.
 */
gboolean irreco_webdb_client_sum(IrrecoWebdbClient *self,
				 gint num_a,
				 gint num_b,
				 glong *sum)
{
	gboolean	 rvalue		= TRUE;
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "sum");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, num_a);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, num_b);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
		    			SOUP_XMLRPC_VALUE_TYPE_INT, self);
	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Get gint (as glong) out of value */
	if(!soup_xmlrpc_value_get_int(value, sum)){
		IRRECO_DEBUG("ERROR: Not proper return value\n");
		g_string_printf(self->error_msg, "ERROR: Not proper return value\n");
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	g_object_unref (response);

	/* Everything went fine, return TRUE */
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_upload_configuration(IrrecoWebdbClient *self,
						  const gchar *backend,
						  const gchar *category,
						  const gchar *file_hash,
						  const gchar *file_name,
						  const gchar *manufacturer,
						  const gchar *model,
						  const gchar *password,
						  const gchar *user,
						  const gchar *data)
{
	gboolean	rvalue		= TRUE;
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Init msg with URI and other data */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "uploadConfiguration");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, backend);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, model);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, user);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, file_hash);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, file_name);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, data);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
		    			SOUP_XMLRPC_VALUE_TYPE_STRING, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	g_object_unref (response);

	/* Everything went fine, return TRUE */
	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_categories(IrrecoWebdbClient *self,
					    IrrecoStringTable **category_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*category_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getCategories");
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*category_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_all_categories(IrrecoWebdbClient *self,
					    IrrecoStringTable **category_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*category_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getAllCategories");
	soup_xmlrpc_message_end_call (msg);

	/** @todo
	 * Upload device dialog also shows category named category
	 * That comes from where and why?
	 * Don't think there should be such
	 * Same thing with manufacturer field
	 */

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*category_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}
gboolean irreco_webdb_client_get_manufacturers(IrrecoWebdbClient *self,
					const gchar *category,
					IrrecoStringTable **manufacturer_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*manufacturer_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getManufacturers");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*manufacturer_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_all_manufacturers(IrrecoWebdbClient *self,
			    IrrecoStringTable **manufacturer_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*manufacturer_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getAllManufacturers");
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("No value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*manufacturer_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_models(IrrecoWebdbClient *self,
					const gchar *category,
					const gchar *manufacturer,
					IrrecoStringTable **model_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*model_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getModels");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("No value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*model_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_configs(IrrecoWebdbClient *self,
					 const gchar *category,
					 const gchar *manufacturer,
					 const gchar *model,
					 IrrecoStringTable **config_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*config_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getConfigurations");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, model);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("No value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("Config: %s\n", ret);

		irreco_string_table_add(*config_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	/* No error occured. */
	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_configuration(IrrecoWebdbClient *self,
					       gint id,
					       IrrecoWebdbConf **configuration)
{
	gboolean	rvalue		= FALSE;
	const gchar	*user = NULL;
	const gchar	*backend;
	const gchar	*category;
	const gchar	*manufacturer;
	const gchar	*model;
	const gchar	*file_hash;
	const gchar	*file_name;
	const gchar	*uploaded;
	const gchar	*download_count;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValue 	*value;
	GHashTable 		*tmp 		= NULL;
	gchar 			*ret 		= NULL;
	SoupXmlrpcValue 	*hash	 	= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*configuration = irreco_webdb_conf_new();

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getConfigurationById");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (long) id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	/* Seek data */
	hash = g_hash_table_lookup (tmp, "user");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	user = ret;

	hash = g_hash_table_lookup (tmp, "backend");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	backend = ret;

	hash = g_hash_table_lookup (tmp, "category");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	category = ret;

	hash = g_hash_table_lookup (tmp, "manufacturer");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	manufacturer = ret;

	hash = g_hash_table_lookup (tmp, "model");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	model = ret;

	hash = g_hash_table_lookup (tmp, "file_hash");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	file_hash = ret;

	hash = g_hash_table_lookup (tmp, "file_name");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	file_name = ret;

	hash = g_hash_table_lookup (tmp, "uploaded");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	uploaded = ret;

	hash = g_hash_table_lookup (tmp, "download_count");
	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}
	download_count = ret;

	IRRECO_DEBUG("Configuration: %d %s %s %s %s %s %s %s %s %s\n",
		     id, user, backend, category, manufacturer,
       		     model, file_hash, file_name, uploaded, download_count);

	irreco_webdb_conf_set(*configuration, id, user, backend, category,
			      manufacturer, model, file_hash, file_name,
			      uploaded, download_count);

	/* No error occured. */
	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}

gint irreco_webdb_client_get_config_id_by_file_hash_and_file_name(
							IrrecoWebdbClient *self,
							const gchar *file_hash,
							const gchar *file_name)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong id = 0;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new(IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call(msg, "getConfigIdByFilehashAndFilename");
	  soup_xmlrpc_message_start_param (msg);
	  soup_xmlrpc_message_write_string(msg, file_hash);
	  soup_xmlrpc_message_write_string(msg, file_name);
	  soup_xmlrpc_message_end_param(msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*)do_xmlrpc(msg,
						  SOUP_XMLRPC_VALUE_TYPE_INT,
						  self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		goto end;
	}

	value = soup_xmlrpc_response_get_value(response);

	/* Try to get integer out of value */
	if(!soup_xmlrpc_value_get_int(value, &id)) {
		id = 0;
	}

	end:
	g_object_unref (response);
	IRRECO_RETURN_INT(id);

}

gboolean irreco_webdb_client_get_file(IrrecoWebdbClient *self,
				      const gchar *file_hash,
				      const gchar *file_name,
				      GString **file_data)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValue 	*value;
	GHashTable 		*tmp 		= NULL;
	gchar 			*ret 		= NULL;
	SoupXmlrpcValue 	*hash	 	= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getFile");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, file_hash);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, file_name);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	/* Get string out from array */
	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	/* Seek data */
	hash = g_hash_table_lookup (tmp, "data");

	if (!soup_xmlrpc_value_get_string(hash, &ret)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	IRRECO_DEBUG("File data:\n%s\n",ret);

	*file_data = g_string_new(ret);

	g_hash_table_destroy (tmp);
	g_object_unref (response);

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_user_exists(IrrecoWebdbClient *self,
					     const gchar *name,
	  				     gboolean *user_exists)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getUserExists");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, name);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
		    			SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get gboolean out of value */
	if(!soup_xmlrpc_value_get_boolean(value, user_exists)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	g_object_unref (response);
	IRRECO_RETURN_BOOL(TRUE);
}

gint irreco_webdb_client_get_max_image_size(IrrecoWebdbClient *self)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong max_image_size = 0;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getMaxImageSize");
	  soup_xmlrpc_message_start_param (msg);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc(msg,
						   SOUP_XMLRPC_VALUE_TYPE_INT,
						   self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		goto end;
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get integer out of value */
	if(!soup_xmlrpc_value_get_int(value, &max_image_size)) {
		max_image_size = 0;
	}

	end:
	g_object_unref (response);
	IRRECO_RETURN_INT(max_image_size);
}

gint irreco_webdb_client_create_theme(IrrecoWebdbClient *self,
				      const gchar *name,
				      const gchar *comment,
				      const gchar *preview_button,
				      const gchar *folder,
				      const gchar *user,
				      const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong theme_id;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "createNewTheme");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, name);
	    soup_xmlrpc_message_write_string(msg, comment);
	    soup_xmlrpc_message_write_string(msg, preview_button);
	    soup_xmlrpc_message_write_string(msg, folder);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_INT, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get integer out of value */
	if(soup_xmlrpc_value_get_int(value, &theme_id)){
		g_object_unref (response);
		IRRECO_RETURN_INT(theme_id);
	}

	g_string_printf(self->error_msg,
		"ERROR: Not proper return value\n");
	g_object_unref (response);
	IRRECO_RETURN_INT(0);
}

gboolean irreco_webdb_client_set_theme_downloadable(IrrecoWebdbClient *self,
						    gint id,
						    gboolean downloadable,
						    const gchar *user,
						    const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	gboolean rvalue = FALSE;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "setThemeDownloadable");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, id);
	    soup_xmlrpc_message_write_boolean(msg, downloadable);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get boolean out of value */
	soup_xmlrpc_value_get_boolean(value, &rvalue);

	g_object_unref (response);
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_themes(IrrecoWebdbClient *self,
					IrrecoStringTable **theme_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*theme_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getThemes");
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*theme_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_theme_by_id(IrrecoWebdbClient *self,
					     gint theme_id,
					     IrrecoWebdbTheme **theme)
{
	gboolean		rvalue		= FALSE;
	gchar			*name		= NULL;
	gchar			*user		= NULL;
	gchar			*comment	= NULL;
	gchar			*preview_button	= NULL;
	gchar			*folder		= NULL;
	gchar			*uploaded	= NULL;
	gchar			*modified	= NULL;
	gchar			*downloaded	= NULL;
	glong			download_count;
	GHashTable 		*tmp 		= NULL;
	SoupXmlrpcValue 	*hash	 	= NULL;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response	= NULL;
	SoupXmlrpcValue 	*value;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getThemeById");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) theme_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		goto end;
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		goto end;
	}

	/* Seek data */
	hash = g_hash_table_lookup(tmp, "name");
	if (!soup_xmlrpc_value_get_string(hash, &name)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "user");
	if (!soup_xmlrpc_value_get_string(hash, &user)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "comment");
	if (!soup_xmlrpc_value_get_string(hash, &comment)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "preview_button");
	if (!soup_xmlrpc_value_get_string(hash, &preview_button)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "folder");
	if (!soup_xmlrpc_value_get_string(hash, &folder)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "uploaded");
	if (!soup_xmlrpc_value_get_string(hash, &uploaded)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "modified");
	if (!soup_xmlrpc_value_get_string(hash, &modified)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "downloaded");
	if (!soup_xmlrpc_value_get_string(hash, &downloaded)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "download_count");
	if (!soup_xmlrpc_value_get_int(hash, &download_count)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	*theme = irreco_webdb_theme_new();

	irreco_webdb_theme_set(*theme, theme_id, name, user, comment,
			       preview_button, NULL, folder, uploaded, modified,
			       downloaded, download_count);

	irreco_webdb_client_get_theme_versions_by_name(self, name,
						       &(*theme)->versions);

	/* Get dates for versions */
	if ((*theme)->versions != NULL)
	{
		IRRECO_STRING_TABLE_FOREACH_KEY((*theme)->versions, key)
			gchar *date = irreco_webdb_client_get_theme_date_by_id(
							self, atoi(key));

			irreco_string_table_change_data ((*theme)->versions,
							 key, (gpointer) date);
		IRRECO_STRING_TABLE_FOREACH_END
	}

	/* No error occured. */
	rvalue = TRUE;

	end:
	if (response != NULL) g_object_unref(response);
	if (tmp != NULL) g_hash_table_destroy(tmp);
	if (name != NULL) g_free(name);
	if (user != NULL) g_free(user);
	if (comment != NULL) g_free(comment);
	if (preview_button != NULL) g_free(preview_button);
	if (folder != NULL) g_free(folder);
	if (uploaded != NULL) g_free(uploaded);
	if (modified != NULL) g_free(modified);
	if (downloaded != NULL) g_free(downloaded);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_theme_versions_by_name(IrrecoWebdbClient *self,
						const char *name,
						IrrecoStringTable **theme_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*theme_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getThemeVersionsByName");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, name);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		goto end;
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			goto end;
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*theme_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;

	end:
	if (rvalue == FALSE) irreco_string_table_free(*theme_list);
	g_object_unref (response);

	IRRECO_RETURN_BOOL(rvalue);
}

gchar *irreco_webdb_client_get_theme_date_by_id(IrrecoWebdbClient *self,
						gint theme_id)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	gchar *date = NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getThemeDateById");
	  soup_xmlrpc_message_start_param (msg);
	  soup_xmlrpc_message_write_int(msg, theme_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*)do_xmlrpc(msg,
						  SOUP_XMLRPC_VALUE_TYPE_STRING,
						  self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		goto end;
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get integer out of value */
	if(!soup_xmlrpc_value_get_string(value, &date)) {
		date = NULL;
	}

	end:
	g_object_unref (response);
	IRRECO_RETURN_PTR(date);
}

gint irreco_webdb_client_get_theme_id_by_name_and_date(IrrecoWebdbClient *self,
						       const gchar *name,
						       const gchar *date)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong id = 0;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getThemeIdByNameAndDate");
	  soup_xmlrpc_message_start_param (msg);
	  soup_xmlrpc_message_write_string(msg, name);
	  soup_xmlrpc_message_write_string(msg, date);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*)do_xmlrpc(msg,
						  SOUP_XMLRPC_VALUE_TYPE_INT,
						  self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		goto end;
	}

	value = soup_xmlrpc_response_get_value(response);

	/* Try to get integer out of value */
	if(!soup_xmlrpc_value_get_int(value, &id)) {
		id = 0;
	}

	end:
	g_object_unref (response);
	IRRECO_RETURN_INT(id);
}

gint irreco_webdb_client_add_bg_to_theme(IrrecoWebdbClient *self,
					 const gchar *name,
					 const gchar *image_hash,
					 const gchar *image_name,
					 const guchar *image,
					 gint image_len,
					 const gchar *folder,
					 gint theme_id,
					 const gchar *user,
					 const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong bg_id;
	gchar *base64_image;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	base64_image = g_base64_encode(image, image_len);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "addBgToTheme");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, name);
	    soup_xmlrpc_message_write_string(msg, image_hash);
	    soup_xmlrpc_message_write_string(msg, image_name);
	    soup_xmlrpc_message_write_string(msg, base64_image);
	    soup_xmlrpc_message_write_string(msg, folder);
	    soup_xmlrpc_message_write_int(msg, theme_id);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	g_free(base64_image);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_INT, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get integer out of value */
	if(soup_xmlrpc_value_get_int(value, &bg_id)){
		g_object_unref (response);
		IRRECO_RETURN_INT(bg_id);
	}

	g_string_printf(self->error_msg,
		"ERROR: Not proper return value\n");
	g_object_unref (response);
	IRRECO_RETURN_INT(0);
}

gboolean irreco_webdb_client_get_backgrounds(IrrecoWebdbClient *self,
					     gint theme_id,
					     IrrecoStringTable **bg_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	glong			ret;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*bg_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getBackgrounds");
	soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, theme_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		gchar *id;
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_int(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			goto end;
		}

		IRRECO_DEBUG("%ld\n", ret);

		id = g_strdup_printf ("%ld", ret);

		irreco_string_table_add(*bg_list, id, NULL);
		g_free(id);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;
	end:
	if(rvalue == FALSE) {
		irreco_string_table_free (*bg_list);
		*bg_list = NULL;
	}
	g_object_unref (response);

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_bg_by_id(IrrecoWebdbClient *self,
					  gint bg_id,
					  const char *theme_bg_dir)
{
	gboolean		rvalue		= FALSE;
	gchar			*name		= NULL;
	gchar			*image_hash	= NULL;
	gchar			*image_name	= NULL;
	gchar			*image_data	= NULL;
	gchar			*base64_image	= NULL;
	gchar			*folder		= NULL;
	GHashTable 		*tmp 		= NULL;
	SoupXmlrpcValue 	*hash	 	= NULL;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValue 	*value;
	GString			*keyfile_path	= g_string_new(theme_bg_dir);
	GString			*image_path	= g_string_new(theme_bg_dir);
	GKeyFile		*keyfile	= g_key_file_new();
	gsize 			base64_len;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getBgById");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) bg_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		goto end;
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		goto end;
	}

	/* Seek data */
	hash = g_hash_table_lookup(tmp, "name");
	if (!soup_xmlrpc_value_get_string(hash, &name)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_hash");
	if (!soup_xmlrpc_value_get_string(hash, &image_hash)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_name");
	if (!soup_xmlrpc_value_get_string(hash, &image_name)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_data");
	if (!soup_xmlrpc_value_get_string(hash, &base64_image)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "folder");
	if (!soup_xmlrpc_value_get_string(hash, &folder)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	/* Create Folder */
	g_string_append_printf(image_path, "/%s", folder);
	IRRECO_DEBUG("mkdir %s\n",image_path->str);
	g_mkdir(image_path->str, 0777);

	/* Save image to folder */
	image_data = (gchar*) g_base64_decode(base64_image, &base64_len);
	g_string_append_printf(image_path, "/%s", image_name);
	irreco_write_file(image_path->str, image_data, base64_len);

	/* Create keyfile and save it to folder*/
	irreco_gkeyfile_set_string(keyfile, "theme-bg" , "name", name);
	irreco_gkeyfile_set_string(keyfile, "theme-bg", "image", image_name);
	g_string_append_printf(keyfile_path, "/%s/bg.conf", folder);
	irreco_write_keyfile(keyfile, keyfile_path->str);

	/* No error occured. */
	rvalue = TRUE;

	end:
	g_object_unref (response);
	if (tmp != NULL) g_hash_table_destroy (tmp);
	if (name != NULL) g_free(name);
	if (image_hash != NULL) g_free(image_hash);
	if (image_name != NULL) g_free(image_name);
	if (image_data != NULL) g_free(image_data);
	if (base64_image != NULL) g_free(base64_image);
	if (folder != NULL) g_free(folder);

	g_key_file_free(keyfile);
	g_string_free(keyfile_path, TRUE);
	g_string_free(image_path, TRUE);

	IRRECO_RETURN_BOOL(rvalue);
}

gint irreco_webdb_client_add_button_to_theme(IrrecoWebdbClient *self,
					     const gchar *name,
					     gboolean allow_text,
					     const gchar *text_format_up,
					     const gchar *text_format_down,
					     gint text_padding,
					     gfloat text_h_align,
					     gfloat text_v_align,
					     const gchar *image_up_hash,
					     const gchar *image_up_name,
					     const guchar *image_up,
					     gint image_up_len,
					     const gchar *image_down_hash,
					     const gchar *image_down_name,
					     const guchar *image_down,
					     gint image_down_len,
					     const gchar *folder,
					     gint theme_id,
					     const gchar *user,
					     const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong button_id;
	gchar *base64_image_up;
	gchar *base64_image_down;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	base64_image_up = g_base64_encode(image_up, image_up_len);
	base64_image_down = g_base64_encode(image_down, image_down_len);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "addButtonToTheme");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, name);
	    soup_xmlrpc_message_write_boolean(msg, allow_text);
	    soup_xmlrpc_message_write_string(msg, text_format_up);
	    soup_xmlrpc_message_write_string(msg, text_format_down);
	    soup_xmlrpc_message_write_int(msg, text_padding);
	    soup_xmlrpc_message_write_double(msg, (gdouble)text_h_align);
	    soup_xmlrpc_message_write_double(msg, (gdouble)text_v_align);
	    soup_xmlrpc_message_write_string(msg, image_up_hash);
	    soup_xmlrpc_message_write_string(msg, image_up_name);
	    soup_xmlrpc_message_write_string(msg, base64_image_up);
	    soup_xmlrpc_message_write_string(msg, image_down_hash);
	    soup_xmlrpc_message_write_string(msg, image_down_name);
	    soup_xmlrpc_message_write_string(msg, base64_image_down);
	    soup_xmlrpc_message_write_string(msg, folder);
	    soup_xmlrpc_message_write_int(msg, theme_id);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	g_free(base64_image_up);
	g_free(base64_image_down);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_INT, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get integer out of value */
	if(soup_xmlrpc_value_get_int(value, &button_id)){
		g_object_unref (response);
		IRRECO_RETURN_INT(button_id);
	}

	g_string_printf(self->error_msg,
		"ERROR: Not proper return value\n");
	g_object_unref (response);
	IRRECO_RETURN_INT(0);
}

gboolean irreco_webdb_client_get_buttons(IrrecoWebdbClient *self,
					 gint theme_id,
					 IrrecoStringTable **button_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	glong			ret;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	*button_list = irreco_string_table_new(NULL, NULL);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getButtons");
	soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, theme_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	while (iter) {
		gchar *id;
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_int(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			goto end;
		}

		IRRECO_DEBUG("%ld\n", ret);

		id = g_strdup_printf("%ld", ret);

		irreco_string_table_add(*button_list, id, NULL);
		g_free(id);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;
	end:
	if(rvalue == FALSE) {
		irreco_string_table_free (*button_list);
		*button_list = NULL;
	}
	g_object_unref (response);

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_button_by_id(IrrecoWebdbClient *self,
					      gint button_id,
					      const char *theme_button_dir)
{
	gboolean		rvalue			= FALSE;
	gchar			*name			= NULL;
	gboolean		allow_text		= FALSE;
	gchar			*text_format_up		= NULL;
	gchar			*text_format_down	= NULL;
	glong			text_padding;
	gdouble			text_h_align;
	gdouble			text_v_align;
	gchar			*image_up_hash		= NULL;
	gchar			*image_up_name		= NULL;
	gchar			*image_up		= NULL;
	gchar			*base64_image_up	= NULL;
	gchar			*image_down_hash	= NULL;
	gchar			*image_down_name	= NULL;
	gchar			*image_down		= NULL;
	gchar			*base64_image_down	= NULL;
	gchar			*folder			= NULL;
	gchar			*image_down_hash_tmp	= NULL;
	GHashTable 		*tmp			= NULL;
	SoupXmlrpcValue 	*hash			= NULL;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValue 	*value;
	GString			*file_path	= g_string_new("");
	GKeyFile		*keyfile	= g_key_file_new();
	gsize 			image_down_len;
	gsize 			image_up_len;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getButtonById");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) button_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		goto end;
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		goto end;
	}

	/* Seek data */
	hash = g_hash_table_lookup(tmp, "name");
	if (!soup_xmlrpc_value_get_string(hash, &name)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "allow_text");
	if (!soup_xmlrpc_value_get_boolean(hash, &allow_text)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "text_format_up");
	soup_xmlrpc_value_get_string(hash, &text_format_up);

	hash = g_hash_table_lookup(tmp, "text_format_down");
	soup_xmlrpc_value_get_string(hash, &text_format_down);

	hash = g_hash_table_lookup(tmp, "text_padding");
	if (!soup_xmlrpc_value_get_int(hash, &text_padding)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "text_h_align");
	if (!soup_xmlrpc_value_get_double(hash, &text_h_align)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}
	hash = g_hash_table_lookup(tmp, "text_v_align");
	if (!soup_xmlrpc_value_get_double(hash, &text_v_align)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_up_hash");
	if (!soup_xmlrpc_value_get_string(hash, &image_up_hash)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_up_name");
	if (!soup_xmlrpc_value_get_string(hash, &image_up_name)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_up");
	if (!soup_xmlrpc_value_get_string(hash, &base64_image_up)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_down_hash");
	if (!soup_xmlrpc_value_get_string(hash, &image_down_hash)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_down_name");
	if (!soup_xmlrpc_value_get_string(hash, &image_down_name)) {
		IRRECO_DEBUG("No value in response\n");
		IRRECO_LINE
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "image_down");
	if (!soup_xmlrpc_value_get_string(hash, &base64_image_down)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup(tmp, "folder");
	if (!soup_xmlrpc_value_get_string(hash, &folder)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	/* Create Folder */
	g_string_printf(file_path, "%s/%s", theme_button_dir, folder);
	IRRECO_DEBUG("mkdir %s\n",file_path->str);
	g_mkdir(file_path->str, 0777);

	/* Save image_up to folder */
	g_string_printf(file_path, "%s/%s/%s", theme_button_dir, folder,
			image_up_name);
	image_up = (gchar*) g_base64_decode(base64_image_up, &image_up_len);
	irreco_write_file(file_path->str, image_up, image_up_len);

	/* Save image_down to folder */
	g_string_printf(file_path, "%s/%s/%s", theme_button_dir, folder,
			image_down_name);

	/*Check image hash data*/
	image_down = (gchar*) g_base64_decode(base64_image_down,
					      &image_down_len);
	irreco_write_file(file_path->str, image_down, image_down_len);
	image_down_hash_tmp = sha_compute_checksum_for_string(G_CHECKSUM_SHA1,
							    image_down,
							    image_down_len);

	if (!g_str_equal(image_down_hash, image_down_hash_tmp)) {

		g_string_printf(self->error_msg,
				"ERROR: Button data is corrupted\n");
		goto end;
	}

	/* Create keyfile and save it to folder*/
	irreco_gkeyfile_set_string(keyfile, "theme-button" , "name", name);

	if (allow_text) {
		irreco_gkeyfile_set_string(keyfile, "theme-button",
					   "allow-text", "true");
	} else {
		irreco_gkeyfile_set_string(keyfile, "theme-button",
					   "allow-text", "false");
	}

	irreco_gkeyfile_set_string(keyfile, "theme-button",
				   "up", image_up_name);

	irreco_gkeyfile_set_string(keyfile, "theme-button",
				   "down", image_down_name);

	if (text_format_up != NULL && strlen(text_format_up) > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme-button",
					"text-format-up", text_format_up);
	}

	if (text_format_down != NULL && strlen(text_format_down) > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme-button",
					"text-format-down", text_format_down);
	}

	irreco_gkeyfile_set_glong(keyfile, "theme-button",
				   "text-padding", (glong)text_padding);

	irreco_gkeyfile_set_gfloat(keyfile,
				   "theme-button",
				   "text-h-align",
				   (gfloat) text_h_align);

	irreco_gkeyfile_set_gfloat(keyfile,
				   "theme-button",
				   "text-v-align",
				   (gfloat) text_v_align);

	g_string_printf(file_path, "%s/%s/button.conf",
			theme_button_dir, folder);
	irreco_write_keyfile(keyfile, file_path->str);

	/* No error occured. */
	rvalue = TRUE;

	end:

	g_object_unref (response);
	if (tmp != NULL) g_hash_table_destroy (tmp);
	if (name != NULL) g_free(name);
	if (text_format_up != NULL) g_free(text_format_up);
	if (text_format_down != NULL) g_free(text_format_down);
	if (image_up_hash != NULL) g_free(image_up_hash);
	if (image_up_name != NULL) g_free(image_up_name);
	if (image_up != NULL) g_free(image_up);
	if (base64_image_up != NULL) g_free(base64_image_up);
	if (image_down_hash != NULL) g_free(image_down_hash);
	if (image_down_name != NULL) g_free(image_down_name);
	if (image_down != NULL) g_free(image_down);
	if (image_down_hash_tmp != NULL) g_free(image_down_hash_tmp);
	if (base64_image_down != NULL) g_free(base64_image_down);
	if (folder != NULL) g_free(folder);

	g_key_file_free(keyfile);
	g_string_free(file_path, TRUE);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_preview_button(IrrecoWebdbClient *self,
						gint theme_id,
						GdkPixbuf **preview_button)
{
	gboolean rvalue = FALSE;
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	char *base64_data = NULL;
	guchar *data;
	gsize len;
        GdkPixbufLoader *pl;
        GError *error = NULL;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getPreviewButton");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, theme_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRING, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		goto end;
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get string out of value */
	if(!soup_xmlrpc_value_get_string(value, &base64_data)){
		g_string_printf(self->error_msg,
		"ERROR: Not proper return value\n");
		goto end;
	}

	data = g_base64_decode(base64_data, &len);

        pl = gdk_pixbuf_loader_new();

        gdk_pixbuf_loader_write(pl, data, len, &error);

        if(error != NULL)
        {
                g_string_printf(self->error_msg, "ERROR: %s", error->message);
                IRRECO_DEBUG("%s\n", error->message);
                goto end;
        }

        gdk_pixbuf_loader_close(pl, NULL);
        *preview_button = gdk_pixbuf_loader_get_pixbuf(pl);

	rvalue = TRUE;

	end:
	g_object_unref (response);
	if (base64_data != NULL) g_free(base64_data);

	IRRECO_RETURN_BOOL(rvalue);
}

static SoupXmlrpcResponse *do_xmlrpc(SoupXmlrpcMessage *xmsg,
				     SoupXmlrpcValueType type,
				     IrrecoWebdbClient *self)
{
	SoupSession *session;
	int status;				/* Status of sent message */
	SoupMessage *spmsg = NULL;		/* Soupmessage */
	SoupXmlrpcResponse *response = NULL;	/* Response */
	SoupXmlrpcValue *value = NULL;		/* Value from response */
	gchar *resbodykeeper;			/* Tmp response holder */
	gboolean error = FALSE;

	IRRECO_ENTER

	session = soup_session_sync_new();   /* Init new synchronous session */

	soup_xmlrpc_message_persist (xmsg);   /* Lock message */

	spmsg = SOUP_MESSAGE(xmsg);   	/* soup XMLRPC msg to soup msg */

	/* Add irreco version to message header */
	soup_message_add_header(spmsg->request_headers,
				"User-Agent", PACKAGE_NAME "/" VERSION);

	IRRECO_DEBUG("Send soup message\n");
	status = soup_session_send_message (session, spmsg);   /* Send msg */

	soup_session_abort (session);	/* Session cleanup */

	g_object_unref (session);

	/* Print request length and response */
	/*IRRECO_DEBUG("\n%.*s\n%d %s\n%.*s\n",
			spmsg->request.length, spmsg->request.body,
			spmsg->status_code, spmsg->reason_phrase,
			spmsg->response.length, spmsg->response.body);*/
	IRRECO_DEBUG("\nRequest length: %d\nStatus code: %d %s\n",
			spmsg->request.length,
			spmsg->status_code, spmsg->reason_phrase);

	/* Check status of sent message */
	if (g_strrstr(spmsg->response.body, "faultCode") != NULL) {

		gchar *errmsg;			/* error message */
		IRRECO_DEBUG("Found faultCode, parse response\n");
		IRRECO_DEBUG("%s\n",spmsg->response.body);

		errmsg = g_malloc0(strlen(spmsg->response.body)*sizeof(gchar));


		/** @todo
		 * Error code and message could be combined to be Error: <num> <msg>
		 * if thats possible to do easily
		 */
		strcpy(errmsg, "Error code: ");

		/* Parse spmsgrspbd to start "<int>" */
		spmsg->response.body = g_strrstr(spmsg->response.body, "<int>");

		resbodykeeper = g_strdup( spmsg->response.body );

		/* Copy part of response to errmsg */
		strcpy(&errmsg[12], &resbodykeeper[5]);

		/* Calc errorcode length and insert ", Error Message: " */
		/* in place of "</int>"  */
		strcpy(&errmsg[strlen(errmsg)-strlen(g_strrstr(errmsg,
					"</int>"))], ", Error Message: ");

		/* Cut 'til "<string>" */
		resbodykeeper = g_strrstr(resbodykeeper, "<string>");

		/* Copy error message from after "<string>" into errmsg */
		strcpy(&errmsg[strlen(errmsg)], &resbodykeeper[8]);

		/* Cut "</string>" and everything after it */
		strcpy(&errmsg[strlen(errmsg)-strlen(g_strrstr(errmsg, "</string>"))], "");

		/* Parsed error message to right variable */
		g_string_printf(self->error_msg, "%s\n", errmsg);

		/* Free errmsg resource */
		g_free(errmsg);
		errmsg = NULL;

		error = TRUE;
		goto cleanup;
	}
	IRRECO_LINE
	/* Handle http failures, eg. 404 */
	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
		IRRECO_DEBUG("HTTP failure\n");
		g_string_printf(self->error_msg, "%d %s\n",
				status, spmsg->reason_phrase);

		error = TRUE;
		goto cleanup;
	}

	/* Parse response */
	response = soup_xmlrpc_response_new();
	response = soup_xmlrpc_message_parse_response (xmsg);

	/* Handle fault responses */
	if (!response) {
		IRRECO_DEBUG("ERROR: no response\n");
		g_string_printf(self->error_msg, "ERROR: no response\n");
		error = TRUE;
		goto cleanup;
	}

	if (soup_xmlrpc_response_is_fault (response)) {
		IRRECO_DEBUG("ERROR: response is fault\n");
		g_string_printf(self->error_msg, "ERROR: response is fault\n");
		error = TRUE;
		goto cleanup;
	}

	/* Get value(s) from response */
	value = soup_xmlrpc_response_get_value(response);

	/* Handle wrong type and no value situations */
	if (!value) {
		IRRECO_DEBUG("ERROR: no value\n");
		g_string_printf(self->error_msg, "ERROR: no value\n");
		error = TRUE;
		goto cleanup;
	}

	if (soup_xmlrpc_value_get_type (value) != type) {
		IRRECO_DEBUG("ERROR: wrong value type\nexpected %s, got %s\n",
			value_type[type],
			value_type[soup_xmlrpc_value_get_type (value)]);

		g_string_printf(self->error_msg,
			"ERROR: wrong value type\nexpected %s, got %s\n",
			value_type[type],
			value_type[soup_xmlrpc_value_get_type (value)]);

		error = TRUE;
		goto cleanup;
	}

	cleanup:
		/* Unref soupmessage */
		if (spmsg != NULL && !error) {
			g_object_unref(spmsg);
			spmsg = NULL;
		}

		if(error) {
			if (response != NULL) g_object_unref(response);
			response = NULL;
		}

	IRRECO_RETURN_PTR(response);
}

gboolean irreco_webdb_client_add_user(IrrecoWebdbClient *self,
				      const gchar *name,
				      const gchar *email,
				      const gchar *passwd)
{
	gboolean	rvalue		= TRUE;

	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	/*gboolean result;*/

	IRRECO_ENTER

	/* Reset error params */
	irreco_webdb_client_reset_env(self);

	/* Init msg with URI and other data */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "addUser");

	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, name);
	  soup_xmlrpc_message_end_param (msg);

	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, email);
	  soup_xmlrpc_message_end_param (msg);

	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, passwd);
	  soup_xmlrpc_message_end_param (msg);

	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
		    			SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	g_object_unref (response);

	/* Everything went fine, return TRUE */
	IRRECO_RETURN_BOOL(rvalue);
}

/**
 * Login user to WebDB.
 *
 */
gboolean irreco_webdb_client_login(IrrecoWebdbClient *self,
						  const gchar *user,
						  const gchar *password)
{
	gboolean	rvalue		= TRUE;
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Init msg with URI and other data */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "loginToDB");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, user);
	  soup_xmlrpc_message_end_param (msg);
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
		    			SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	g_object_unref (response);

	/* Everything went fine, return TRUE */
	IRRECO_RETURN_BOOL(rvalue);
}

gint irreco_webdb_client_create_new_remote(IrrecoWebdbClient *self,
					   const gchar *comment,
					   const gchar *category,
					   const gchar *manufacturer,
					   const gchar *model,
					   const gchar *file_name,
					   const gchar *file_data,
					   const gchar *user,
					   const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	glong remote_id;
	gchar *file_hash;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	file_hash = sha_compute_checksum_for_string(G_CHECKSUM_SHA1,
						    file_data, -1);
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "createNewRemote");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, comment);
	    soup_xmlrpc_message_write_string(msg, category);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	    soup_xmlrpc_message_write_string(msg, model);
	    soup_xmlrpc_message_write_string(msg, file_hash);
	    soup_xmlrpc_message_write_string(msg, file_name);
	    soup_xmlrpc_message_write_string(msg, file_data);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_INT, self);

	g_free(file_hash);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_INT(0);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get integer out of value */
	if(soup_xmlrpc_value_get_int(value, &remote_id)){
		g_object_unref (response);
		IRRECO_RETURN_INT(remote_id);
	}

	g_string_printf(self->error_msg, "ERROR: Not proper return value\n");
	g_object_unref (response);
	IRRECO_RETURN_INT(0);
}

gboolean irreco_webdb_client_set_remote_downloadable(IrrecoWebdbClient *self,
						     gint id,
						     gboolean downloadable,
						     const gchar *user,
						     const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	gboolean rvalue = FALSE;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "setRemoteDownloadable");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, id);
	    soup_xmlrpc_message_write_boolean(msg, downloadable);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get boolean out of value */
	soup_xmlrpc_value_get_boolean(value, &rvalue);

	g_object_unref (response);
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_add_configuration_to_remote(
							IrrecoWebdbClient *self,
							gint remote_id,
							gint configuration_id,
							const gchar *user,
							const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	gboolean rvalue = FALSE;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "addConfigurationToRemote");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, remote_id);
	    soup_xmlrpc_message_write_int(msg, configuration_id);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get boolean out of value */
	soup_xmlrpc_value_get_boolean(value, &rvalue);

	g_object_unref (response);
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_add_theme_to_remote(IrrecoWebdbClient *self,
						 gint remote_id,
						 gint theme_id,
						 const gchar *user,
						 const gchar *password)
{
	SoupXmlrpcMessage *msg;
	SoupXmlrpcResponse *response;
	SoupXmlrpcValue *value;
	gboolean rvalue = FALSE;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "addThemeToRemote");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, remote_id);
	    soup_xmlrpc_message_write_int(msg, theme_id);
	    soup_xmlrpc_message_write_string(msg, user);
	    soup_xmlrpc_message_write_string(msg, password);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_BOOLEAN, self);

	/* Check response exists */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	value = soup_xmlrpc_response_get_value (response);

	/* Try to get boolean out of value */
	soup_xmlrpc_value_get_boolean(value, &rvalue);

	g_object_unref (response);
	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remote_categories(IrrecoWebdbClient *self,
					    IrrecoStringTable **category_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemoteCategories");
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*category_list = irreco_string_table_new((GDestroyNotify)
						 irreco_string_table_free,
						 NULL);
	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			irreco_string_table_free(*category_list);
			*category_list = NULL;
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*category_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;
	g_object_unref(response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remote_manufacturers(IrrecoWebdbClient *self,
					const gchar *category,
					IrrecoStringTable **manufacturer_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemoteManufacturers");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*manufacturer_list = irreco_string_table_new((GDestroyNotify)
						     irreco_string_table_free,
						     NULL);

	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			irreco_string_table_free(*manufacturer_list);
			*manufacturer_list = NULL;
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*manufacturer_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;
	g_object_unref(response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remote_models(IrrecoWebdbClient *self,
					       const gchar *category,
					       const gchar *manufacturer,
					       IrrecoStringTable **model_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemoteModels");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*model_list = irreco_string_table_new((GDestroyNotify)
					      irreco_string_table_free, NULL);
	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("No value\n");
			g_object_unref (response);
			irreco_string_table_free(*model_list);
			*model_list = NULL;
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*model_list, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;
	g_object_unref(response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remote_creators(IrrecoWebdbClient *self,
						 const gchar *category,
						 const gchar *manufacturer,
						 const gchar *model,
						 IrrecoStringTable **creators)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	gchar 			*ret 		= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemoteCreators");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	    soup_xmlrpc_message_write_string(msg, model);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value(response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*creators = irreco_string_table_new((GDestroyNotify) g_list_free, NULL);
	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_string(array_val, &ret)) {
			IRRECO_DEBUG ("No value\n");
			g_object_unref(response);
			irreco_string_table_free(*creators);
			*creators = NULL;
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%s\n", ret);

		irreco_string_table_add(*creators, ret, NULL);

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	rvalue = TRUE;
	g_object_unref(response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remotes(IrrecoWebdbClient *self,
					 const gchar *category,
					 const gchar *manufacturer,
					 const gchar *model,
					 const gchar *creator,
					 GList **remote_list)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	glong			ret;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemotes");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_string(msg, category);
	    soup_xmlrpc_message_write_string(msg, manufacturer);
	    soup_xmlrpc_message_write_string(msg, model);
	    soup_xmlrpc_message_write_string(msg, creator);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value(response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*remote_list = NULL;
	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_int(array_val, &ret)) {
			IRRECO_DEBUG ("No value\n");
			g_object_unref(response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%d\n", (gint) ret);

		*remote_list = g_list_append(*remote_list,
					     GINT_TO_POINTER(ret));

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	*remote_list = g_list_first(*remote_list);

	rvalue = TRUE;
	g_object_unref(response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remote_by_id(IrrecoWebdbClient *self,
					      gint id,
					      IrrecoWebdbRemote **remote)
{
	gboolean		rvalue		= FALSE;
	gchar			*user		= NULL;
	gchar			*comment	= NULL;
	gchar			*category	= NULL;
	gchar			*manufacturer	= NULL;
	gchar			*model		= NULL;
	gchar			*file_hash	= NULL;
	gchar			*file_name	= NULL;
	gchar			*uploaded	= NULL;
	gchar			*modified	= NULL;
	gchar			*downloaded	= NULL;
	glong			download_count;
	SoupXmlrpcMessage 	*msg		= NULL;
	SoupXmlrpcResponse 	*response	= NULL;
	SoupXmlrpcValue 	*value		= NULL;
	GHashTable 		*tmp 		= NULL;
	SoupXmlrpcValue 	*hash	 	= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemoteById");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		goto end;
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		goto end;
	}

	/* Seek data */
	hash = g_hash_table_lookup (tmp, "user");
	if (!soup_xmlrpc_value_get_string(hash, &user)) {
		IRRECO_DEBUG("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "comment");
	if (!soup_xmlrpc_value_get_string(hash, &comment)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "category");
	if (!soup_xmlrpc_value_get_string(hash, &category)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "manufacturer");
	if (!soup_xmlrpc_value_get_string(hash, &manufacturer)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "model");
	if (!soup_xmlrpc_value_get_string(hash, &model)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "file_hash");
	if (!soup_xmlrpc_value_get_string(hash, &file_hash)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "file_name");
	if (!soup_xmlrpc_value_get_string(hash, &file_name)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "uploaded");
	if (!soup_xmlrpc_value_get_string(hash, &uploaded)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "modified");
	if (!soup_xmlrpc_value_get_string(hash, &modified)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "downloaded");
	if (!soup_xmlrpc_value_get_string(hash, &downloaded)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	hash = g_hash_table_lookup (tmp, "download_count");
	if (!soup_xmlrpc_value_get_int(hash, &download_count)) {
		IRRECO_DEBUG ("No value in response\n");
		goto end;
	}

	*remote = irreco_webdb_remote_new();
	irreco_webdb_remote_set(*remote, id, user, comment, category,
				manufacturer, model, file_hash, file_name,
				uploaded, modified, downloaded, download_count);

	/* No error occured. */
	rvalue = TRUE;

	end:
	if (response != NULL) g_object_unref(response);
	if (tmp != NULL) g_hash_table_destroy(tmp);
	if (user != NULL) g_free(user);
	if (comment != NULL) g_free(comment);
	if (category != NULL) g_free(category);
	if (manufacturer != NULL) g_free(manufacturer);
	if (model != NULL) g_free(model);
	if (file_hash != NULL) g_free(file_hash);
	if (file_name != NULL) g_free(file_name);
	if (uploaded != NULL) g_free(uploaded);
	if (modified != NULL) g_free(modified);
	if (downloaded != NULL) g_free(downloaded);

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_themes_of_remote(IrrecoWebdbClient *self,
						  gint remote_id,
						  GList **themes)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	glong			ret;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getThemesOfRemote");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) remote_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*themes = NULL;
	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_int(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%d\n", (gint) ret);

		*themes = g_list_append(*themes, GINT_TO_POINTER(ret));

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	*themes = g_list_first(*themes);

	rvalue = TRUE;
	g_object_unref (response);

	IRRECO_RETURN_BOOL(rvalue);
}


gboolean irreco_webdb_client_get_configurations_of_remote(
						  IrrecoWebdbClient *self,
						  gint remote_id,
						  GList **configs)
{
	gboolean		rvalue		= FALSE;
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValueArrayIterator *iter;
	SoupXmlrpcValue 	*value;
	SoupXmlrpcValue 	*array_val;
	glong			ret;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getConfigurationsOfRemote");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) remote_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_ARRAY, self);
	if(!response) {
		IRRECO_DEBUG(" No response, failed something\n");
		IRRECO_RETURN_BOOL(rvalue);
	}

	value = soup_xmlrpc_response_get_value (response);

	soup_xmlrpc_value_array_get_iterator(value, &iter);

	*configs = NULL;
	while (iter) {
		soup_xmlrpc_value_array_iterator_get_value(iter, &array_val);

		if (!soup_xmlrpc_value_get_int(array_val, &ret)) {
			IRRECO_DEBUG ("NO  value\n");
			g_object_unref (response);
			IRRECO_RETURN_BOOL(rvalue);
		}
		IRRECO_DEBUG("%d\n", (gint) ret);

		*configs = g_list_append(*configs, GINT_TO_POINTER(ret));

		iter = soup_xmlrpc_value_array_iterator_next(iter);
	}

	*configs = g_list_first(*configs);

	rvalue = TRUE;
	g_object_unref (response);

	IRRECO_RETURN_BOOL(rvalue);
}

gboolean irreco_webdb_client_get_remote_data(IrrecoWebdbClient *self,
					     gint remote_id,
					     gchar **file_data)
{
	SoupXmlrpcMessage 	*msg;
	SoupXmlrpcResponse 	*response;
	SoupXmlrpcValue 	*value;
	GHashTable 		*tmp 		= NULL;
	SoupXmlrpcValue 	*hash	 	= NULL;

	IRRECO_ENTER

	irreco_webdb_client_reset_env(self);

	/* Build query */
	msg = soup_xmlrpc_message_new (IRRECO_WEBDB_URL);
	soup_xmlrpc_message_start_call (msg, "getRemoteData");
	  soup_xmlrpc_message_start_param (msg);
	    soup_xmlrpc_message_write_int(msg, (glong) remote_id);
	  soup_xmlrpc_message_end_param (msg);
	soup_xmlrpc_message_end_call (msg);

	/* Execute XML-RPC call. */
	response = (SoupXmlrpcResponse*) do_xmlrpc (msg,
					SOUP_XMLRPC_VALUE_TYPE_STRUCT, self);

	/* Check response */
	if(!response) {
		IRRECO_DEBUG(" No response, failed at do_xmlrpc\n");
		IRRECO_RETURN_BOOL(FALSE);
	}

	/* Get array out from response */
	value = soup_xmlrpc_response_get_value (response);

	/* Get string out from array */
	if(!soup_xmlrpc_value_get_struct(value, &tmp)){
		g_string_printf(self->error_msg,
				"ERROR: Not proper return value\n");
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	/* Seek data */
	hash = g_hash_table_lookup (tmp, "data");

	if (!soup_xmlrpc_value_get_string(hash, file_data)) {
		IRRECO_DEBUG ("No value in response\n");
		g_hash_table_destroy (tmp);
		g_object_unref (response);
		IRRECO_RETURN_BOOL(FALSE);
	}

	IRRECO_DEBUG("File data:\n%s\n", *file_data);

	g_hash_table_destroy (tmp);
	g_object_unref(response);

	IRRECO_RETURN_BOOL(TRUE);
}

/** @} */


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */
